require 'time'
require 'panoramix/plugin/base'
require 'highline/import'
require 'toml'

module Panoramix
  module Plugin

    class CloudFormationExceptionError < StandardError; end

    class CloudFormation < Base

      attr_reader :dst
      attr_reader :src
      attr_reader :stage
      attr_reader :params
      attr_reader :owner
      attr_reader :version

      def initialize(dst, src, parameters, stage, owner, version)
        @dst = dst
        @src = src
        @stage = stage
        @owner = owner
        @version = version

        @params = TOML.load_file(parameters)
      end

      # Final Cloud Formation stack name
      def gen_stack_name
        @version ? "#{@stage}-#{@dst}-#{@version}-#{owner}" : "#{@stage}-#{@dst}-#{owner}"
      end

      # Return stack name
      def stack_name
        puts gen_stack_name
      end

      def ask_pro
        choose do |menu|
          menu.prompt = "Would you like to delete the stack #{gen_stack_name}? "

          menu.choice(:No)  do
            say("Don't do it again!")
            return false
          end
          menu.choices(:Yes) do
            resp = ask("Warning, this action can not be reversed. Do you want to continue? (Yes/No)".red, String) do |q|
              q.default = "No"
            end
            return resp == "Yes"
          end
        end
      end

      def question
        case @stage
        when "pro"
          return true if ENV['CFN_FORCE_DELETE']
          return ask_pro
        when "pre"
          return true if ENV['CFN_FORCE_DELETE']
          return ask_pro
        when "test"
          return true
        when "dev"
          return true
        else
          return false
        end
      end

      # Return the stack creation time
      def timestamp
        return Time.at(0) unless @created

        return Time.parse(@created["CreationTime"])
      end

      # Has this stacks already been created
      def created?
        # Get a list af all created Stacks
        query = shell("aws cloudformation describe-stacks --query 'Stacks[*].{StackName:StackName, CreationTime:CreationTime}'", true)[:out]
        parsed_stacks = JSON.parse query

        # Check wether the stacks has already been created
        info = parsed_stacks.select { |s| s["StackName"] == gen_stack_name }
        @created = info.empty? ? nil: info.first
      end

      # Action delete for this task
      def delete
        return unless created?
        return unless question

        cmd = "aws cloudformation delete-stack \
  --stack-name #{gen_stack_name}"

        shell(cmd)
        # Wait until the stack is being deleted
        loop do
          break if !created?
        end
      end

      # When this instance needs to be executed
      def needed? timestamps
        this_time = timestamp
        timestamps.any? { |t| t > this_time }
      end

      # Match every parameter with an environment variable
      def fill_env path
        file = File.read path
        json = JSON.parse file

        env = Hash.new
        json["Parameters"].each do |k,v|
          env[k] = @params["default"][k] ? @params["default"][k] : nil
          env[k] = @params[@stage][k] ? @params[@stage][k] : env[k]
          env[k] = ENV[k] ? ENV[k] : env[k]
          raise "Parameter or environment variable #{k} not defined" unless env[k]
        end
        env
      end

      # Wait until the stack is being created
      def wait_for_creation stack_id
        loop do
          status = shell("aws cloudformation describe-stacks --stack #{stack_id} --query Stacks[0].StackStatus --output text", true)[:out].strip
          raise "Creation Failed" if status == "CREATE_FAILED"
          break if status != "CREATE_IN_PROGRESS"
          sleep 30
        end
      end

      # Check wether the file exists
      def validate_file path
        # Raise error if the file does not exist
        unless File.exists? path
          message = I18n.t('errors.cfn.template_not_found', {:path => path})
          raise(Panoramix::Plugin::DockerUpExceptionError, message)
        end
      end

      # Deafult task
      def run_default
        if created?
          puts "WARNING: #{gen_stack_name} already exists!".red
          return
        end
        env = fill_env @src

        params = ""
        env.each {|k,v| params = params + "ParameterKey=#{k},ParameterValue=#{v} "}

        validate_file @src
        template = "file://#{@src}"

        cmd = "aws cloudformation create-stack \
  --query StackId \
  --stack-name #{gen_stack_name} \
  --template-body #{template} \
  --disable-rollback \
  --parameters #{params}"

        result = shell(cmd, true)[:out]
        stack_id = result.strip

        wait_for_creation stack_id
      end

      def ps
        if created?
          shell("aws cloudformation describe-stacks --stack-name #{gen_stack_name}")
        end
      end
    end

  end
end

def cfn(args, &block)
  name = args.keys[0]

  deps = args[name]

  cfn_options = deps.select { |d| d.is_a? Hash }

  # Get cloudformation template file path
  template = cfn_options.empty? ? nil : cfn_options.first[:template]

  # Raise error if :template value is nil
  unless template
    message = I18n.t('errors.cfn.no_template_provided')
    raise(Panoramix::Plugin::DockerUpExceptionError, message)
  end

  # Required by aws cli
  template = File.expand_path template

  # Get cloudformation parameters file path
  parameters = cfn_options.empty? ? nil : cfn_options.first[:parameters]

  # Raise error if :parameters value is nil
  unless parameters
    message = I18n.t('errors.cfn.no_parameters_provided')
    raise(Panoramix::Plugin::DockerUpExceptionError, message)
  end

  # Raise error if the file does not exist
  unless File.exists? parameters
    message = I18n.t('errors.cfn.parameters_not_found', {:path => parameters})
    raise(Panoramix::Plugin::DockerUpExceptionError, message)
  end

  # Get the PANORAMIX_OWNER environment variable
  owner = ENV["PANORAMIX_OWNER"]
  raise "Variable PANORAMIX_OWNER not defined" unless owner

  # Convert name.surname to namesurname
  owner = owner.gsub(".", "")

  descriptions = I18n.t('cfn')
  descriptions = Hash.new if descriptions.class != Hash

  # Merge deps with the cloudformation template file task
  prerequisites = deps.select { |d| ! d.is_a? Hash }
  prerequisites = prerequisites.push(template)

  # Create a task for each version
  ["blue", "green", nil].each do |version|
    # Create a task inside each development stage namespace
    ["pro", "pre", "dev", "test"].each do |stage|
      stage_name = version ? "#{stage}:#{name}:#{version}" : "#{stage}:#{name}"
      instance = Panoramix::Plugin::CloudFormation.new(name, template, parameters, stage, owner, version)
      Panoramix.define_tasks(stage_name, descriptions, instance, prerequisites, block)
    end
  end
end

=begin
["blue", "green"].each do |version|
 ["pro", "pre", "dev", "test"].each do |stage|
 end
end
=end
