require "time"
require "panoramix/plugin/base"

module Panoramix
	module Plugin

		class DockerUpExceptionError < StandardError; end

		class DockerUp < Base

			attr_reader :dst
			attr_reader :src
			attr_reader :env

			def initialize(dst, src, env, host)
				@dst = dst
				@src = src
				@env = env

				@env = parse_env @env.keys
				@env["DOCKER_HOST"] = "tcp://#{host}" unless host.nil?
			end

			# Return current timestamp for the container
			def timestamp
				info = created?
				if info
					id = info.split("\n").first.strip
					time = shell("docker inspect -f {{.Created}} #{id}", true, @env)[:out]
					return Time.parse(time)
			  else
			  	Time.at 0
			  end
			end

			# Has this image already been created
			def created?
				info = shell("docker-compose -p #{@dst} -f #{@src} ps -q", true, @env)[:out]
				@created = info = info.empty? ? nil: info
			end

			# Action rm for this task
			def rm
				clobber
			end

			# Action clobber for this task
			def clobber
				shell("docker-compose -p #{@dst} -f #{@src} kill; docker-compose -p #{@dst} -f #{@src} rm -v --force", false, @env) if created?
			end

			# When this instance needs to be executed
			def needed? timestamps
				this_time = timestamp
				timestamps.any? { |t| t > this_time }
			end

			# Default action for this task
			def run_default
				# Raise an exception if already has been created
				if created?
					message = I18n.t('errors.docker_up.cluster_deployed', {:id => @dst})
					raise(DockerUpExceptionError, message)
				end
	      # Run docker-compose up otherwise
		    shell("docker-compose -p #{dst} -f #{@src}  up -d", false, @env)
		  end

			def stop
				if created?
					shell("docker-compose -p #{dst} -f #{@src} stop", false, @env)
				end
			end

			def start
				if created?
					shell("docker-compose -p #{dst} -f #{@src}  up -d", false, @env)
				end
			end

			def id
				info = created?
				if info
					containers = info.split("\n")
					puts containers
				end
			end

		  # Print docker-compose project logs
	    def logs
				info = created?
				if info
					containers = info.split("\n")
					colors = ['red', 'green','yellow','blue','magenta','cyan']
					containers.each do |c|
						out = shell("docker logs #{c} | tail -n 200", true, @env)
						name = shell("docker inspect -f {{.Name}} #{c}", true, @env)[:out]
						name.gsub!("\n", "")
						name.gsub!("/", "")
						name = eval %& "#{name} >".#{colors.first}&
						colors.push colors.shift
						out[:out].split("\n").each{ |out| puts name + out }
						out[:err].split("\n").each{ |out| puts name + out }
					end
				end
	    end

	    # Print docker-compose ps
	    def ps
	    	puts "Service: #{@dst}"
				if created?
          puts "Timestamp: #{timestamp}"
          info = shell("docker-compose -p #{@dst} -f #{@src} ps", true, @env)[:out]
		    	puts info
				else
					puts "Timestamp: Not created"
				end
				puts
	    end
		end

	end
end


def parse_env environment
  res=Hash.new

  environment.each do |var|
    res[var]=ENV[var]
  end

  res
end

def docker_up(args, &block)
	name = args.keys[0]

	deps = args[name]

	compose_params = deps.select { |d| d.is_a? Hash }

	# Get docker-compose file path
	compose = compose_params.first[:compose]

	# Raise error if :compose value is nil
	unless compose
		message = I18n.t('errors.docker_up.no_compose_provided')
		raise(Panoramix::Plugin::DockerUpExceptionError, message)
	end

	# Get flags
	flags = {}

	# Get environment
	environment = parse_env(compose_params.first[:env] || [])

	 # Select the host
  host = compose_params[0].fetch(:host, nil)

	# Merge deps with the compose file_task
	prerequisites = deps.select { |d| ! d.is_a? Hash }
	prerequisites = prerequisites.push(compose)

	instance = Panoramix::Plugin::DockerUp.new(name, compose, environment, host)

	descriptions = I18n.t('docker_up')
	descriptions = Hash.new if descriptions.class != Hash

	Panoramix.define_tasks(name, descriptions, instance, prerequisites, block)
end
